package sun.awt;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.Window;
import java.net.URL;
import java.util.Hashtable;
import netscape.applet.AppletServices;
import sun.awt.image.ByteArrayImageSource;
import sun.awt.image.FileImageSource;
import sun.awt.image.URLImageSource;

public abstract class SunToolkit extends Toolkit {
   protected static EventQueue theEventQueue;
   protected static final Hashtable peerMap = new Hashtable();
   static Hashtable imgHash = new Hashtable();

   public SunToolkit() {
      String var1 = Toolkit.getProperty("AWT.EventQueueClass", "java.awt.EventQueue");

      try {
         theEventQueue = (EventQueue)Class.forName(var1).newInstance();
      } catch (Exception var3) {
         System.err.println("Failed loading " + var1 + ": " + var3);
         theEventQueue = new EventQueue();
      }
   }

   protected static Object targetToPeer(Object var0) {
      return var0 != null ? peerMap.get(var0) : null;
   }

   protected static void targetDisposedPeer(Object var0, Object var1) {
      if (var0 != null && var1 != null && peerMap.get(var0) == var1) {
         peerMap.remove(var0);
      }

   }

   public Dimension getScreenSize() {
      return new Dimension(this.getScreenWidth(), this.getScreenHeight());
   }

   protected abstract int getScreenWidth();

   protected abstract int getScreenHeight();

   public String[] getFontList() {
      String[] var1 = new String[]{"Dialog", "SansSerif", "Serif", "Monospaced", "Helvetica", "TimesRoman", "Courier", "DialogInput", "ZapfDingbats"};
      return var1;
   }

   static synchronized Image getImageFromHash(Toolkit var0, URL var1) {
      SecurityManager var2 = System.getSecurityManager();
      if (var2 != null) {
         var2.checkConnect(var1.getHost(), var1.getPort());
      }

      Image var3 = (Image)imgHash.get(var1);
      if (var3 == null) {
         try {
            var3 = var0.createImage(new URLImageSource(var1));
            imgHash.put(var1, var3);
         } catch (Exception var4) {
         }
      }

      return var3;
   }

   static synchronized Image getImageFromHash(Toolkit var0, String var1) {
      SecurityManager var2 = System.getSecurityManager();
      if (var2 != null) {
         var2.checkRead(var1);
      }

      Image var3 = (Image)imgHash.get(var1);
      if (var3 == null) {
         try {
            var3 = var0.createImage(new FileImageSource(var1));
            imgHash.put(var1, var3);
         } catch (Exception var4) {
         }
      }

      return var3;
   }

   public Image getImage(String var1) {
      return getImageFromHash(this, (String)var1);
   }

   public Image getImage(URL var1) {
      return getImageFromHash(this, (URL)var1);
   }

   public Image createImage(byte[] var1, int var2, int var3) {
      return ((Toolkit)this).createImage(new ByteArrayImageSource(var1, var2, var3));
   }

   protected EventQueue getSystemEventQueueImpl() {
      EventQueue var1 = EventQueue.getQueue(Thread.currentThread().getThreadGroup());
      if (var1 == null) {
         var1 = theEventQueue;
      }

      return var1;
   }

   public static Container getNativeContainer(Component var0) {
      return Toolkit.getNativeContainer(var0);
   }

   public static Window[] getTopLevelWindows() {
      ThreadGroup var0 = Thread.currentThread().getThreadGroup();
      return AppletServices.getTopLevelWindows(var0);
   }

   public static Dialog swapTopModalDialog(Dialog var0) {
      ThreadGroup var1 = Thread.currentThread().getThreadGroup();
      return AppletServices.swapTopModalDialog(var0, var1);
   }
}
